@@ -1,6 +1,5 @@
-(* $I1: Unison file synchronizer: src/fspath.mli $ *)
-(* $I2: Last modified by vouillon on Wed, 26 May 2004 09:43:22 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/fspath.mli *)
+(* Copyright 1999-2009, Benjamin C. Pierce (see COPYING for details) *)
 
 (* Defines an abstract type of absolute filenames (fspaths)                  *)
 
@@ -13,11 +12,6 @@ val canonize : string option -> t
 val toString : t -> string
 val concatToString : t -> Path.local -> string
 
-(* If the string is the name of the local root, returns a localPath such     *)
-(* that concatenated to the root, it points to the same file as              *)
-(* (fspath, path) in this replica.                                           *)
-val fullLocalPath : string -> t -> Path.local -> Path.local
-
 (* If fspath+path refers to a (followed) symlink, then return the directory  *)
 (* of the symlink's target; otherwise return the parent dir of path.  If     *)
 (* fspath+path is a root directory, raise Fatal.                             *)
